function [dzcutf hfout status message]=NDACC_ResolDFbis(dzsampling, coef, hfinp)

% BELOW ARE SUGGESTED FINAL COMMENTS FOR DISTRIBUTION (PLEASE CHECK AND REVISE AS APPROPRIATE)

% This routine computes vertical resolution using the ISSI-Team-agreed standardized defintion based on the cutoff frequency of digitial filters.
% For details, please refer to ISSI Team Report at http://www.issibern.ch/teams/ndacc/ISSI_Team_Report.htm
% IMPORTANT: If several filtering events occur during data processing, this routine must be called each time a filtering event occurs.
%            The values of "hfout" returned during the first call must be used for "hfinp" in the second call, and so on until the last call.
%            The returned vertical resolution "dzcutf" of the last call only needs to be used (no need for those computed during previosu calls)

% File history:
% Created in 2011 by Thierry Leblanc, NASA-Jet Propulsion Laboratory, thierry.leblanc@jpl.nasa.gov
% Revised in June 2015 by T. Leblanc: Introduce factor of 2 ( i.e., 1/(2*fc) instead of 1/fc ) to add consistency with impulse response definition
% Revised in Nov. 2015 by T. Leblanc: Introduce input parameter "dzsampling" and use it to compute vertical resolution in physical unit (last line of routine)

% Inputs:
%     dzsampling = Vertical sampling resolution of signal/profile being filtered (in physical unit)
%     coef       = A vector of 2*N+1 coefficients containing the values of the filter coefficients used to smooth or differentiate the signal/profile
%                  The coefficients must be either symmetric (c(k)=c(-k) for all k), or anti-symmteric (c(k)=-c(-k) for all k)
%     hfinp      = A vector (suggested length = total length of your signal/profile) containing the values of the transfer function in the frequency domain.
%                  If routine first call, it is equal to 1 for all frequencies
%                  If not first call, it must be equal to the transfer function "hfout" returned by the routine during the previous call
% Outputs:
%     status  = Informaiton flag on computaiton status: 1 = succes (smoothing) , 2 = sucess (differentiation) , 0 = failure (cancel)
%     dzcutf  = The value of vertical resolution using the standardized defintion based on the FWHM of an impulse response
%               (same unit as the input sampling resolution)
%     hfout   = A vector of same lenght as hfinp containing the values of the calculated transfer function used to compute the cutoff frequency.
%               If this routine is called again later, use this output as the input transfer function "hfinp" for the next call
%     message = A string containing an error message (if status=0)

% BELOW ARE THE ORIGNAL COMMENTS BY GUILLAUME? FRANCK?

% Calculation of the normalized resolution of the filter. To obtain the
% real resolution, multiplie dzcutf by the initial resolution. 
%
% [dzcutf hfout status message]=NDACC_ResolDF(dzsampling,coef) or 
% [dzcutf hfout status message]=NDACC_ResolDF(dzsampling,coef, []): calculate the
% Resolution of the filter whose coefficients are "coef" ("coef"
% must be an odd array of value)
%
% [dzcutf hfout status message]=NDACC_ResolDF(dzsampling, coef, hfinp): hfinp is the
% transfer function of the last filter. The length of hfout is the same of
% hfinp.
%
% INPUTS:
%   - dzsampling[scalar]: sampling resolution (physical unit)
%   - coef [array of number]: coefficients of the filter
%   - hfinp [array of number]: [optional] transfer function of last filter
%
% OUTPUT:
%   - dzcutf [array of number]: resolution of the filter with unit equal to that of dzsampling 
%   - hfout [array of number]: transfer function of system (this filter and
%   last filters)
%   - status [Value]: 
%       - 0: problem
%       - 1: low-pass filter
%       - 2: derivative filter
%   - message [String]: error message (if status=0)

%% Parameters
missval = -99;
message='';
dzcutf=missval;
hfout=missval;
nf=4096;
nfmin=512;

%% Checking Inputs
% nomber of input
if nargin<1
    status=0;
    message='[ERROR] NDACC_ResolDF.m: Not enought inputs for this function';
    return
end
% Checking coef
if isempty(coef)
    status=0;
    message='[ERROR] NDACC_ResolDF.m: coef must be no empty';
    return
end
if mod(coef, 2)==0
    status=0;
    message='[ERROR] NDACC_ResolDF.m: length of coef must be odd';
    return
end
if length(coef)==1 && coef(1)~=1
    status=0;
    message='[ERROR] NDACC_ResolDF.m: if the length of coef is 1, this value must be 1';
    return
end
if size(coef, 1)==1
    coef=coef';
end
% Checking hfinp
if exist('hfinp', 'var')
    if ~isempty(hfinp)
        if size(hfinp,1)==1
            hfinp=hfinp';
        end
        nf=length(hfinp);
        if nf<nfmin
            disp(['[WARNING] NDACC_ResolDF.m: the length of hfinp is low: it ' ...
                'is better to take a length of ' num2str(nfmin) ' in minimum'])
        end
    else
        hfinp=ones(nf,1);
    end
else
    hfinp=ones(nf,1);
end
hfout=ones(nf, 1).*missval;

%% Symetry of coefficients
m=(length(coef)-1)/2;
if coef(1:m)+coef(end:-1:m+2)<1e-5
    status=2;
else
    if coef(1:m)-coef(end:-1:m+2)<1e-5
        status=1;
    else
        status=0;
        message='[ERROR] NDACC_ResolDF.m: coefficients must be symmetric or anti-symmetric';
        return
    end
end

%% Creation of the frequency vector
f(1,:)=0.5/nf:0.5/nf:0.5;

%% Calculation of the transfer function of the filter
hfcomp=zeros(length(f),1);  
for j=m+1:length(f)-m
    if status==1
        hfcomp(j,1)=coef(m+1)+2*cos(2*pi*f(j).*(1:m))*coef(m+2:end);
    else
        hfcomp(j,1)=-2*sin(2*pi*f(j).*(1:m))*coef(m+2:end)/(2*pi*f(j));
    end
end
hfcomp=abs(hfcomp);

%% Limit conditions
hfcomp(1:m,1)=hfcomp(m+1,1);
hfcomp(end-m+1,end)=hfcomp(end-m,1);

%% Final transfer function
hfout=hfcomp.*hfinp;

%% Identify fc (cutoff frequency) i.e. hfout(fc)=0.5
i_fc=find(hfout<0.5,1)-1;
if ~isempty(i_fc)
    if i_fc~=0
        a=(f(i_fc)-f(i_fc+1))/(hfout(i_fc)-hfout(i_fc+1));
        fc=f(i_fc+1)+a*(0.5-hfout(i_fc+1));
    else
        fc=0.5;
    end
else
    fc=0.5;
end
dzcutf=dzsampling/(2*fc);

